package clime.messadmin.admin;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import clime.messadmin.utils.Base64;

/**
 * @author C&eacute;drik LIME
 */
class HTTPAuthorizationProvider {

	private static byte[] md5(String message) {
		try {
			MessageDigest md = MessageDigest.getInstance("MD5");//$NON-NLS-1$
			return md.digest(message.getBytes());
		} catch (NoSuchAlgorithmException nsae) {
			throw new RuntimeException("Error while computing md5 hash: " + nsae.getLocalizedMessage());//$NON-NLS-1$
		}
	}


	public static boolean checkAccess(final String authorizationPassword, HttpServletRequest request, HttpServletResponse response) throws IOException {
		// no password set => no authorization required
		if (authorizationPassword == null || "".equals(authorizationPassword.trim())) {//$NON-NLS-1$
			return true;
		}

		// password from a previously-set authorization cookie?
		if (request.getCookies() != null) {
			Cookie[] cookies = request.getCookies();
			String authorizationPasswordHash = Base64.encodeToString(md5(authorizationPassword), false);
			// some browsers always set null for cookie.getPath()...
			String cookieName = "MessAdmin" + request.getContextPath();//$NON-NLS-1$
			for (int i = 0; i < cookies.length; ++i) {
				Cookie cookie = cookies[i];
				if (cookieName.equals(cookie.getName())) {
					String providedPasswordHash = cookie.getValue();
					if (authorizationPasswordHash.equals(providedPasswordHash)) {
						return true;
					}
				}
			}
		}

		// we must authenticate the user before letting her play with us
		String providedPassword = null;
		if (request.getParameter("password") != null) {//$NON-NLS-1$
			// password in URL
			providedPassword = request.getParameter("password");//$NON-NLS-1$
			//request.setAttribute("password", providedPassword);//$NON-NLS-1$
		} else if (request.getHeader("Authorization") != null) {//$NON-NLS-1$
			// password from HTTP Access Authentication
			String authorization = request.getHeader("Authorization");//$NON-NLS-1$
			if (! authorization.startsWith("Basic ")) {//$NON-NLS-1$
				//TODO we should use "Digest" instead of "Basic", but it is more complicated to code...
				throw new IllegalArgumentException("Only Basic HTTP Access Authentication supported");//$NON-NLS-1$
			}
			String base64UserPass = authorization.substring("Basic ".length()).trim();//$NON-NLS-1$
//			String userPass = new String(org.apache.commons.codec.binary.Base64.decodeBase64(base64UserPass.getBytes()));
			String userPass = new String(Base64.decode(base64UserPass.getBytes()));
			int index = userPass.indexOf(':');
			//String user = userPass.substring(0, index);
			String password = userPass.substring(index+1);
			providedPassword = password;
		} else {
			providedPassword = null;
		}

		if (authorizationPassword.equals(providedPassword)) {
			// set authorization Cookie
			// some browsers always set null for cookie.getPath()...
			String cookieName = "MessAdmin" + request.getContextPath();//$NON-NLS-1$
			Cookie cookie = new Cookie(cookieName, Base64.encodeToString(md5(authorizationPassword), false));
			cookie.setVersion(1);
			cookie.setMaxAge(-1);
			cookie.setPath(request.getContextPath());
			response.addCookie(cookie);
			return true;
		} else {
			// request Authorization Password
			//TODO we should use "Digest" instead of "Basic", but it is more complicated to code...
			response.setHeader("WWW-Authenticate", "Basic realm=\"MessAdmin Administration for " + request.getContextPath() + '"');
			response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
			return false;
		}
	}
}
