/**
 * 
 */
package clime.messadmin.providers.spi;

import java.util.ResourceBundle;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import clime.messadmin.i18n.I18NSupport;
import clime.messadmin.model.Server;


/**
 * Base implementation class for DataProvider displaying tabular data.
 * @author C&eacute;drik LIME
 */
public abstract class BaseTabularDataProvider {

	/**
	 * 
	 */
	public BaseTabularDataProvider() {
		super();
	}

	/**
	 * Convenience method for i18n
	 * @since 4.1
	 */
	protected ClassLoader getClassLoader(final HttpSession session) {
		return Server.getInstance().getApplication(session.getServletContext()).getApplicationInfo().getClassLoader();
	}
	/**
	 * Convenience method for i18n
	 * @since 4.1
	 */
	protected ClassLoader getClassLoader(final ServletContext context) {
		return Server.getInstance().getApplication(context).getApplicationInfo().getClassLoader();
	}

	/**
	 * Convenience method for i18n
	 * @since 4.1
	 */
	protected ResourceBundle getResourceBundle(String baseName, ClassLoader cl) {
		return I18NSupport.getResourceBundle(baseName, cl);
	}

	/**
	 * Convenience method for i18n
	 * @since 4.1
	 */
	protected String getLocalizedMessage(String baseName, ClassLoader cl, String key) {
		return I18NSupport.getLocalizedMessage(baseName, cl, key);
	}
	/**
	 * Convenience method for i18n
	 * @since 4.1
	 */
	protected String getLocalizedMessage(String baseName, ClassLoader cl, String key, Object[] args) {
		return I18NSupport.getLocalizedMessage(baseName, cl, key, args);
	}

	//	/**
//	 * @param object
//	 * @return specific data labels for given object, or null if it can be determined
//	 */
//	public abstract String[] getTabularDataLabels(final Object object);
//	/**
//	 * @param object
//	 * @return specific data values for given object, or null if it can be determined
//	 */
//	public abstract Object[][] getTabularData(final Object object);
//
//	protected abstract String getTableCaption(String[] labels, Object[][] data);
//
//	protected String getXHTMLData(Object object) {
//		try {
//			String[] labels = getTabularDataLabels(object);
//			Object[][] values = getTabularData(object);
//			return buildXHTML(labels, values, tableId, getTableCaption(labels, values));
//		} catch (RuntimeException rte) {
//			return "Error in " + this.getClass().getName() + ": " + rte;
//		}
//	}

	protected String buildXHTML(String[] labels, Object[][] values, String tableId, String tableCaption) {
		if (labels == null || values == null || values.length == 0) {
			return "";
		}
		StringBuffer buffer = new StringBuffer(128 + 16*labels.length + 16*values.length*labels.length);
		buffer.append("<table id=\"").append(tableId).append("\" class=\"strippable\" style=\"text-align: left;\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\">\n");
		appendCaption(buffer, tableCaption);
		appendHeader(buffer, labels);
		appendFooter(buffer, labels);
		appendBody(buffer, values);
		buffer.append("</table>\n");
		return buffer.toString();
	}

	protected void appendCaption(StringBuffer buffer, String tableCaption) {
		buffer.append("<caption style=\"font-variant: small-caps;\">").append(tableCaption).append("</caption>\n");
	}

	protected void appendHeader(StringBuffer buffer, String[] labels) {
		buffer.append("<thead><tr>\n");
		for (int i = 0; i < labels.length; ++i) {
			String label = labels[i];
			buffer.append("<th>");
			appendHeaderLabel(buffer, label);
			buffer.append("</th>");
		}
		buffer.append("\n</tr></thead>\n");
	}

	protected void appendHeaderLabel(StringBuffer buffer, String label) {
		if (label != null) {
			buffer.append(label);
		}
	}

	protected void appendBody(StringBuffer buffer, Object[][] values) {
		buffer.append("<tbody>\n");
		for (int i = 0; i < values.length; ++i) {
			Object[] row = values[i];
			appendRow(buffer, row);
		}
		buffer.append("</tbody>\n");
	}

	protected void appendRow(StringBuffer buffer, Object[] row) {
		buffer.append("<tr>\n");
		for (int j = 0; j < row.length; ++j) {
			Object value = row[j];
			buffer.append("<td>");
			appendValue(buffer, value);
			buffer.append("</td>");
		}
		buffer.append("</tr>\n");
	}

	protected void appendValue(StringBuffer buffer, Object value) {
		if (value != null) {
//			buffer.append("<span title=\"").append(value.getClass()).append("\">");
			buffer.append(value);
//			buffer.append("</span>");
		}
	}

	protected void appendFooter(StringBuffer buffer, String[] labels) {
//		buffer.append("<tfoot><tr>\n");
//		for (int i = 0; i < labels.length; ++i) {
//			String label = labels[i];
//			buffer.append("<th>");
//			appendFooterLabel(buffer, label);
//			buffer.append("</th>");
//		}
//		buffer.append("\n</tr></tfoot>\n");
	}

	protected void appendFooterLabel(StringBuffer buffer, String label) {
		appendHeaderLabel(buffer, label);
	}
}
