/**
 * 
 */
package clime.messadmin.providers.spi;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import clime.messadmin.model.IApplicationInfo;
import clime.messadmin.model.IServerInfo;
import clime.messadmin.model.ISessionInfo;
import clime.messadmin.providers.ProviderUtils;

/**
 * Provider for displaying the administration data.
 * Sample implementation can be HTML, text, CSV...
 * @author C&eacute;drik LIME
 * @since 4.1
 */
public interface DisplayFormatProvider extends BaseProvider {
	public static class Util {
		public static String FORMAT_PARAMETER_NAME = "format";//$NON-NLS-1$
		public static String DEFAULT_FORMAT = "html";//$NON-NLS-1$
		public static DisplayFormatProvider getInstance(HttpServletRequest request) {
			String format = request.getParameter(FORMAT_PARAMETER_NAME);
			if (format == null) {
				format = DEFAULT_FORMAT;
			}
			return getInstance(format);
		}
		public static DisplayFormatProvider getInstance(String format) {
			Iterator iter = ProviderUtils.getProviders(DisplayFormatProvider.class).iterator();
			while (iter.hasNext()) {
				DisplayFormatProvider provider = (DisplayFormatProvider) iter.next();
				if (format.equalsIgnoreCase(provider.getFormatID())) {
					return provider;
				}
			}
			throw new IllegalArgumentException(format);
		}
		public static void initAll(ServletConfig config) throws ServletException {
			Iterator iter = ProviderUtils.getProviders(DisplayFormatProvider.class).iterator();
			while (iter.hasNext()) {
				DisplayFormatProvider provider = (DisplayFormatProvider) iter.next();
				provider.init(config);
			}
		}
	}

	public String getFormatID();

	public String getContentType();

	public void init(ServletConfig config) throws ServletException;

	/**
	 * @param req
	 * @param resp
	 * @throws ServletException
	 * @throws IOException
	 */
	public void preProcess(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException;

	/**
	 * Displays the Web Applications list
	 * @param req
	 * @param resp
	 * @param applicationInfos
	 * @throws ServletException
	 * @throws IOException
	 */
	public void displayWebAppsList(HttpServletRequest req, HttpServletResponse resp,
			Set/*<IApplicationInfo>*/ applicationInfos) throws ServletException, IOException;
	/**
	 * Displays the HttpSessions list for a given WebApp (context)
	 * @param req
	 * @param resp
	 * @param sortBy
	 * @param orderBy
	 * @param webAppStats
	 * @param activeSessions
	 * @param passiveSessionsIds
	 * @throws ServletException
	 * @throws IOException
	 */
	public void displaySessionsListPage(HttpServletRequest req, HttpServletResponse resp,
			String sortBy, String orderBy,
			IApplicationInfo webAppStats, Collection/*<ISessionInfo>*/ activeSessions, Collection/*<String>*/ passiveSessionsIds)
			throws ServletException, IOException;
	/**
	 * Displays the details page for a given HttpSession
	 * @param req
	 * @param resp
	 * @param webAppStats
	 * @param currentSession
	 * @throws ServletException
	 * @throws IOException
	 */
	public void displaySessionDetailPage(HttpServletRequest req, HttpServletResponse resp,
			IApplicationInfo webAppStats, ISessionInfo currentSession) throws ServletException, IOException;
	/**
	 * Displays the details page for a Web Application
	 * @param req
	 * @param resp
	 * @param webAppStats
	 * @throws ServletException
	 * @throws IOException
	 */
	public void displayWebAppStatsPage(HttpServletRequest req, HttpServletResponse resp,
			IApplicationInfo webAppStats) throws ServletException, IOException;
	/**
	 * Displays the page for the Server Informations
	 * @param req
	 * @param resp
	 * @param serverInfo
	 * @throws ServletException
	 * @throws IOException
	 */
	public void displayServerInfosPage(HttpServletRequest req, HttpServletResponse resp,
			IServerInfo serverInfo) throws ServletException, IOException;
}
