package clime.messadmin.utils;

import java.net.MalformedURLException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import clime.messadmin.model.ISessionInfo;
import clime.messadmin.model.Server;

/**
 * Utility methods on HttpSessions...
 * @author C&eacute;drik LIME
 */
public class SessionUtils {

	/**
	 * 
	 */
	private SessionUtils() {
		super();
	}


	public static int getUsedTimeForSession(HttpSession in_session) {
		try {
			ISessionInfo extraSessionInfo = Server.getInstance().getSession(in_session).getSessionInfo();
			if (null != extraSessionInfo) {
				return extraSessionInfo.getTotalUsedTime();
			} else {
				return -1;
			}
			//long diffMilliSeconds = in_session.getLastAccessedTime() - in_session.getCreationTime();
			//return diffMilliSeconds;
		} catch (IllegalStateException ise) {
			//ignore: invalidated session
			return -1;
		}
	}

	public static int getTTLForSession(HttpSession in_session) {
		try {
			long diffMilliSeconds = (1000*in_session.getMaxInactiveInterval()) - (System.currentTimeMillis() - in_session.getLastAccessedTime());
			return (int) diffMilliSeconds;
		} catch (IllegalStateException ise) {
			//ignore: invalidated session
			return -1;
		}
	}

	public static int getIdleTimeForSession(HttpSession in_session) {
		try {
			long diffMilliSeconds =  System.currentTimeMillis() - in_session.getLastAccessedTime();
			return (int) diffMilliSeconds;
		} catch (IllegalStateException ise) {
			//ignore: invalidated session
			return -1;
		}
	}

	//TODO: move this method in a utility package

	/**
	 * Returns the context path of the web application. The context path is
	 * the portion of the request URI that is used to select the context of
	 * the request. The context path always comes first in a request URI. The
	 * path starts with a "/" character but does not end with a "/" character.
	 * For servlets in the default (root) context, this method returns "".
	 * 
	 * It is possible that a servlet container may match a context by more than
	 * one context path. In such cases getContextPath() will return the actual
	 * context path used by the request and it may differ from the path returned
	 * by this method. The context path returned by this method should be considered
	 * as the prime or preferred context path of the application.
	 * 
	 * @return The context path of the web application.
	 */
	/**
	 * FIXME debug this method so that it really works! Should return the same as HttpServletRequest.getContextPath(). Now, where the hell is ServletContext.getContextPath()? :-( UPDATE: it's in 2.5 now!
	 * Returns the portion of the request URI that indicates the context
	 * of the request.  The context path always comes first in a request
	 * URI.  The path starts with a "/" character but does not end with a "/"
	 * character.  For servlets in the default (root) context, this method
	 * returns "". The container does not decode this string.
	 * 
	 * It is possible that a servlet container may match a context by more
	 * than one context path. In such cases this method will return the actual
	 * context path used by the request and it may differ from the path returned
	 * by the ServletContext.getContextPath() method. The context path returned
	 * by ServletContext.getContextPath() should be considered as the prime or
	 * preferred context path of the application.
	 * 
	 * @param session
	 * @return	a <code>String</code> specifying the portion of
	 *			the request URI that indicates the context of the request
	 */
	public static String getContext(HttpSession session) {
		return getContext(session.getServletContext());
	}
	public static String getContext(ServletContext context) {
		//FIXME try new 2.5 ServletContext.getContextPath() first; use new plugin architecture
		try {
			return context.getResource("/").getPath();//$NON-NLS-1$
		} catch (MalformedURLException mue) {
			throw new RuntimeException(mue.getMessage());
		}
	}

	/**
	 * Reconstructs the URL the client used to make the request,
	 * using information in the <code>HttpServletRequest</code> object.
	 * The returned URL contains a protocol, server name, port
	 * number, and server path, and include query
	 * string parameters.
	 * 
	 * <p>This method is useful for creating redirect messages
	 * and for reporting errors.
	 *
	 * @param req	a <code>HttpServletRequest</code> object
	 *			containing the client's request
	 * 
	 * @return		a <code>String</code> object containing
	 *			the reconstructed URL
	 */
	public static String getRequestURLWithMethodAndQueryString(HttpServletRequest req) {
		/*
		StringBuffer url = new StringBuffer(32);
		String scheme = req.getScheme();
		int port = req.getServerPort();
		if (port < 0) {
			port = 80; // Work around java.net.URL bug
		}
		//String	servletPath = req.getServletPath();
		//String 	pathInfo = req.getPathInfo();
		String queryString = req.getQueryString();

		url.append(scheme);	// http, https
		url.append("://"); //$NON-NLS-1$
		url.append(req.getServerName());
		if ((scheme.equals ("http") && port != 80) //$NON-NLS-1$
			|| (scheme.equals ("https") && port != 443)) { //$NON-NLS-1$
			url.append (':'); //$NON-NLS-1$
			url.append (port);
		}
		//if (servletPath != null)
		//	url.append (servletPath);
		//if (pathInfo != null)
		//	url.append (pathInfo);
		url.append(req.getRequestURI());
		if (queryString != null) {
			url.append('?').append(queryString); //$NON-NLS-1$
		}
		return url.toString();
		*/
		String method = req.getMethod();
		StringBuffer requestURL = req.getRequestURL();
		String queryString = req.getQueryString();
		int totalLength = method.length() + 1 + requestURL.length() + (queryString==null?0:1+queryString.length());
		StringBuffer buffer = new StringBuffer(totalLength);
		buffer.append(method).append(' ').append(requestURL);
		if (queryString != null && !"".equals(queryString)) {//$NON-NLS-1$
			buffer.append('?').append(queryString);
		}
		return buffer.toString();
	}

}
